import 'package:flutter/material.dart';
import 'package:flutter_widget_from_html_core/flutter_widget_from_html_core.dart';
import 'package:go_router/go_router.dart';
import 'package:logging/logging.dart';

import '../../globals.dart';
import '../../models/flattened_tree_item.dart';
import '../../models/timeline_entry.dart';
import '../../services/timeline_manager.dart';
import '../../utils/active_profile_selector.dart';
import '../../utils/clipboard_utils.dart';
import '../../utils/url_opening_utils.dart';
import '../media_attachment_viewer_control.dart';
import '../padding.dart';
import 'interactions_bar_control.dart';
import 'link_preview_control.dart';
import 'status_header_control.dart';

class FlattenedTreeEntryControl extends StatefulWidget {
  final FlattenedTreeItem originalItem;
  final bool openRemote;
  final bool showStatusOpenButton;

  const FlattenedTreeEntryControl(
      {super.key,
      required this.originalItem,
      required this.openRemote,
      required this.showStatusOpenButton});

  @override
  State<FlattenedTreeEntryControl> createState() => _StatusControlState();
}

class _StatusControlState extends State<FlattenedTreeEntryControl> {
  static final _logger = Logger('$FlattenedTreeEntryControl');

  var showContent = true;

  var showComments = false;

  FlattenedTreeItem get item => widget.originalItem;

  TimelineEntry get entry => item.timelineEntry;

  bool get isPost => entry.parentId.isEmpty;

  bool get hasComments => entry.engagementSummary.repliesCount > 0;

  bool isProcessing = false;

  @override
  void initState() {
    showContent = entry.spoilerText.isEmpty;
    showComments = isPost ? false : true;
  }

  @override
  Widget build(BuildContext context) {
    _logger.finest('Building ${entry.toShortString()}');
    const otherPadding = 8.0;
    final leftPadding = otherPadding + (widget.originalItem.level * 15.0);
    final color = widget.originalItem.level.isOdd
        ? Theme.of(context).secondaryHeaderColor
        : Theme.of(context).dialogBackgroundColor;
    final body = Container(
      decoration: BoxDecoration(
        color: color,
        border: Border.all(width: 0.5),
        borderRadius: BorderRadius.circular(5.0),
        boxShadow: [
          BoxShadow(
            color: Theme.of(context).dividerColor,
            blurRadius: 2,
            offset: Offset(4, 4),
            spreadRadius: 0.1,
            blurStyle: BlurStyle.normal,
          )
        ],
      ),
      child: Padding(
        padding: const EdgeInsets.all(5.0),
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Row(
              children: [
                Expanded(
                  child: StatusHeaderControl(
                    entry: entry,
                  ),
                ),
                buildMenuControl(context),
              ],
            ),
            const VerticalPadding(
              height: 5,
            ),
            if (entry.spoilerText.isNotEmpty)
              TextButton(
                  onPressed: () {
                    setState(() {
                      showContent = !showContent;
                    });
                  },
                  child: Text(
                      'Content Summary: ${entry.spoilerText} (Click to ${showContent ? "Hide" : "Show"}}')),
            if (showContent) ...[
              buildBody(context),
              const VerticalPadding(
                height: 5,
              ),
              if (entry.linkPreviewData != null)
                LinkPreviewControl(preview: entry.linkPreviewData!),
              buildMediaBar(context),
            ],
            const VerticalPadding(
              height: 5,
            ),
            InteractionsBarControl(
              entry: entry,
              isMine: item.isMine,
              showOpenControl: widget.showStatusOpenButton,
            ),
            const VerticalPadding(
              height: 5,
            ),
          ],
        ),
      ),
    );
    return Padding(
      padding: EdgeInsets.only(
        left: leftPadding,
        right: otherPadding,
        top: otherPadding,
        bottom: otherPadding,
      ),
      child: body,
    );
  }

  Widget buildBody(BuildContext context) {
    return HtmlWidget(
      entry.body,
      onTapUrl: (url) async {
        return await openUrlStringInSystembrowser(context, url, 'link');
      },
    );
  }

  Widget buildMediaBar(BuildContext context) {
    final items = entry.mediaAttachments;
    if (items.isEmpty) {
      return const SizedBox();
    }
    return SizedBox(
        height: 250.0,
        child: ListView.separated(
            scrollDirection: Axis.horizontal,
            itemBuilder: (context, index) {
              return MediaAttachmentViewerControl(
                attachments: items,
                index: index,
              );
            },
            separatorBuilder: (context, index) {
              return HorizontalPadding();
            },
            itemCount: items.length));
  }

  Widget buildMenuControl(BuildContext context) {
    const editStatus = 'Edit';
    const deleteStatus = 'Delete';
    const goToPost = 'Open Post';
    const copyText = 'Copy Post Text';
    const copyUrl = 'Copy URL';
    const openExternal = 'Open In Browser';
    final options = [
      if (widget.showStatusOpenButton && !widget.openRemote) goToPost,
      if (item.isMine && !item.timelineEntry.youReshared) editStatus,
      if (item.isMine) deleteStatus,
      copyText,
      openExternal,
      copyUrl,
    ];

    return PopupMenuButton<String>(onSelected: (menuOption) async {
      if (!mounted) {
        return;
      }

      switch (menuOption) {
        case goToPost:
          context.push(
              '/post/view/${item.timelineEntry.id}/${item.timelineEntry.id}');
          break;
        case editStatus:
          if (item.timelineEntry.parentId.isEmpty) {
            context.push('/post/edit/${item.timelineEntry.id}');
          } else {
            context.push('/comment/edit/${item.timelineEntry.id}');
          }
          break;
        case deleteStatus:
          deleteEntry();
          break;
        case openExternal:
          await openUrlStringInSystembrowser(
            context,
            item.timelineEntry.externalLink,
            'Post',
          );
          break;
        case copyUrl:
          await copyToClipboard(
            context: context,
            text: item.timelineEntry.externalLink,
            message: 'Post link copied to clipboard',
          );
          break;
        case copyText:
          await copyToClipboard(
            context: context,
            text: item.timelineEntry.body,
            message: 'Post link copied to clipboard',
          );
          break;
        default:
        //do nothing
      }
    }, itemBuilder: (context) {
      return options
          .map((o) => PopupMenuItem(value: o, child: Text(o)))
          .toList();
    });
  }

  Future<void> deleteEntry() async {
    setState(() {
      isProcessing = true;
    });
    final confirm =
        await showYesNoDialog(context, 'Delete ${isPost ? "Post" : "Comment"}');
    if (confirm == true) {
      await getIt<ActiveProfileSelector<TimelineManager>>()
          .activeEntry
          .andThenAsync(
              (tm) async => await tm.deleteEntryById(item.timelineEntry.id));
    }
    setState(() {
      isProcessing = false;
    });
  }
}
