import 'package:flutter/material.dart';
import 'package:go_router/go_router.dart';
import 'package:logging/logging.dart';
import 'package:provider/provider.dart';

import '../controls/app_bottom_nav_bar.dart';
import '../controls/current_profile_button.dart';
import '../controls/linear_status_indicator.dart';
import '../controls/standard_app_drawer.dart';
import '../globals.dart';
import '../models/connection.dart';
import '../routes.dart';
import '../services/connections_manager.dart';
import '../services/network_status_service.dart';
import '../utils/active_profile_selector.dart';

class ContactsScreen extends StatefulWidget {
  @override
  State<ContactsScreen> createState() => _ContactsScreenState();
}

class _ContactsScreenState extends State<ContactsScreen> {
  static final _logger = Logger('$ContactsScreen');
  var filterText = '';

  @override
  Widget build(BuildContext context) {
    final nss = getIt<NetworkStatusService>();
    final manager = context
        .watch<ActiveProfileSelector<ConnectionsManager>>()
        .activeEntry
        .value;
    final allContacts = manager.getMyContacts();
    final filterTextLC = filterText.toLowerCase();
    final contacts = allContacts
        .where((c) =>
            filterText.isEmpty ||
            c.name.toLowerCase().contains(filterTextLC) ||
            c.handle.toLowerCase().contains(filterTextLC))
        .toList();
    contacts.sort((c1, c2) => c1.name.compareTo(c2.name));
    _logger.finer(
      () => '# Contacts: ${allContacts.length}, #filtered: ${contacts.length}',
    );
    late Widget body;
    if (contacts.isEmpty) {
      body = SingleChildScrollView(
          physics: AlwaysScrollableScrollPhysics(),
          child: Center(
            child: Text('No contacts'),
          ));
    } else {
      body = ListView.separated(
          physics: const AlwaysScrollableScrollPhysics(),
          itemBuilder: (context, index) {
            final contact = contacts[index];
            return ListTile(
              onTap: () {
                context.pushNamed(ScreenPaths.userProfile,
                    params: {'id': contact.id});
              },
              title: Text(contact.name),
              trailing: Text(contact.status.label()),
            );
          },
          separatorBuilder: (context, index) => const Divider(),
          itemCount: contacts.length);
    }

    return SafeArea(
      child: Scaffold(
        appBar: AppBar(
          leading: buildCurrentProfileButton(context),
          title: TextField(
            onChanged: (value) {
              setState(() {
                filterText = value.toLowerCase();
              });
            },
            decoration: InputDecoration(
              labelText: 'Filter By Name',
              alignLabelWithHint: true,
              border: OutlineInputBorder(
                borderSide: BorderSide(
                  color: Theme.of(context).backgroundColor,
                ),
                borderRadius: BorderRadius.circular(5.0),
              ),
            ),
          ),
        ),
        drawer: StandardAppDrawer(),
        body: SafeArea(
          child: RefreshIndicator(
            onRefresh: () async {
              if (nss.connectionUpdateStatus.value) {
                return;
              }
              manager.updateAllContacts();
              return;
            },
            child: Column(
              children: [
                StandardLinearProgressIndicator(nss.connectionUpdateStatus),
                Expanded(child: body),
              ],
            ),
          ),
        ),
        bottomNavigationBar: AppBottomNavBar(
          currentButton: NavBarButtons.contacts,
        ),
      ),
    );
  }
}
